/**
* @file      Strapp API and initialization driver
* @author    Jordan Lavatai, Ken Grimes
* @version   0.0.4
* @license   AGPL-3.0
* @copyright August 2017 - Ken Grimes, Jordan Lavatai
* @summmary  
*/
import fs from "./strappFileSystem.js"

const strapp = (() => {
  const events = {}
  return {
    request: (location, method, data) => new Promise((resolve, reject) => {
      fs.request(location, method, data)
        .then((response) => resolve(response))
        .catch((err) => reject(err))
    }),
    on: (eventName, fn) => {
      if (!events[eventName])
        events[eventName] = []
      events[eventName].push(fn)
    },
    emit: (eventName, data) =>
      events[eventName] && events[eventName].forEach((fn) => fn(data))
    ,
    GET: (location) => fs.request(location, 'GET'),
    POST: (location, data) => fs.request(location, 'POST', data)
  }
  //!set up root account (/acct/local)
  //.make device pseudo-files (/dev/keyboard, /dev/touch, /dev/camera, /dev/audio, ...)
  //-multiplexed devices:  /run/<pid>/keyboard, /run/<pid>/touch, /run/<pid>
  //                      (directory whose post/gets are sockets to app)
  //devices are just web api event forwarders.  e.g.:
  //document.addEventListener('keydown') etcetera -> /dev/keyboard.emit('keydown')
  //CONNECT adds remote listener?  CONNECT gives event emitter (listener) for file
  //  that emits GET/PUT/POST/DEL/etc events.  each file has a "connections" list,
  //  which are objects with .send methods (sockets, or an event emitter object
  //  evt emitter object has a fn emit(evt), and .send(evt) -> emit(evt)
})()

export default strapp
